import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_svg/flutter_svg.dart';

class SVGAsset extends SvgPicture {
  SVGAsset(
    String assetName, {
    Key? key,
    Color? color,
    double? size,
    double? height,
    double? width,
    bool matchTextDirection = false,
    AssetBundle? bundle,
    String? package,
    BoxFit fit = BoxFit.contain,
    AlignmentGeometry alignment = Alignment.center,
    bool allowDrawingOutsideViewBox = false,
    WidgetBuilder? placeholderBuilder,
    BlendMode colorBlendMode = BlendMode.srcIn,
    String? semanticsLabel,
    bool excludeFromSemantics = false,
    Clip clipBehavior = Clip.hardEdge,
    bool cacheColorFilter = false,
    SvgTheme? theme,
  }) : super.asset(assetName,
            key: key,
            color: color,
            width: width ?? size,
            height: height ?? size,
            matchTextDirection: matchTextDirection,
            bundle: bundle,
            package: package,
            fit: fit,
            alignment: alignment,
            allowDrawingOutsideViewBox: allowDrawingOutsideViewBox,
            placeholderBuilder: placeholderBuilder,
            colorBlendMode: colorBlendMode,
            semanticsLabel: semanticsLabel,
            excludeFromSemantics: excludeFromSemantics,
            clipBehavior: clipBehavior,
            cacheColorFilter: cacheColorFilter,
            theme: theme);
}

class SVGFile extends SvgPicture {
  SVGFile(
    File file, {
    Key? key,
    Color? color,
    double? size,
    double? height,
    double? width,
    bool matchTextDirection = false,
    AssetBundle? bundle,
    String? package,
    BoxFit fit = BoxFit.contain,
    AlignmentGeometry alignment = Alignment.center,
    bool allowDrawingOutsideViewBox = false,
    WidgetBuilder? placeholderBuilder,
    BlendMode colorBlendMode = BlendMode.srcIn,
    String? semanticsLabel,
    bool excludeFromSemantics = false,
    Clip clipBehavior = Clip.hardEdge,
    bool cacheColorFilter = false,
    SvgTheme? theme,
  }) : super.file(file,
            key: key,
            color: color,
            width: width ?? size,
            height: height ?? size,
            matchTextDirection: matchTextDirection,
            fit: fit,
            alignment: alignment,
            allowDrawingOutsideViewBox: allowDrawingOutsideViewBox,
            placeholderBuilder: placeholderBuilder,
            colorBlendMode: colorBlendMode,
            semanticsLabel: semanticsLabel,
            excludeFromSemantics: excludeFromSemantics,
            clipBehavior: clipBehavior,
            cacheColorFilter: cacheColorFilter,
            theme: theme);
}

class SVGNetwork extends SvgPicture {
  SVGNetwork(
    String url, {
    Key? key,
    Color? color,
    double? size,
    double? height,
    double? width,
    bool matchTextDirection = false,
    AssetBundle? bundle,
    String? package,
    BoxFit fit = BoxFit.contain,
    AlignmentGeometry alignment = Alignment.center,
    bool allowDrawingOutsideViewBox = false,
    WidgetBuilder? placeholderBuilder,
    BlendMode colorBlendMode = BlendMode.srcIn,
    String? semanticsLabel,
    bool excludeFromSemantics = false,
    Clip clipBehavior = Clip.hardEdge,
    bool cacheColorFilter = false,
    SvgTheme? theme,
  }) : super.network(url,
            key: key,
            color: color,
            width: width ?? size,
            height: height ?? size,
            matchTextDirection: matchTextDirection,
            fit: fit,
            alignment: alignment,
            allowDrawingOutsideViewBox: allowDrawingOutsideViewBox,
            placeholderBuilder: placeholderBuilder,
            colorBlendMode: colorBlendMode,
            semanticsLabel: semanticsLabel,
            excludeFromSemantics: excludeFromSemantics,
            clipBehavior: clipBehavior,
            cacheColorFilter: cacheColorFilter,
            theme: theme);
}
